procedure Biconnected(var (* for speed *) G: graph;
                      var ... (* output mechanism for components *) );
  (* The procedure finds all biconnected components of a connected
     undirected graph which consist of at least one edge. *)

  type ExtendedVertex = 0 .. MaxVertex; (* 0 is a null parent. *)
  var dfsnumber, highwater: array [vertex] of 0 .. MaxVertex;
      lastdfsnumber: 0 .. MaxVertex;
      i: vertex;

  (* Program assumes a stack package (tailored to edges) with operations
       procedure CreateStack, procedure Push(v, w: vertex),
       and procedure Pop(var v, w: vertex). *)

  procedure BiconnectedComponent(v (* vertex *): vertex;
                                 u (* parent *): ExtendedVertex);
    var p: PtrToNode;
        w, a, b: vertex;
    begin
      (* Set up the bookkeeping for v. *)
      lastdfsnumber := lastdfsnumber + 1;
      dfsnumber[v] := lastdfsnumber;
      highwater[v] := lastdfsnumber;
      (* Traverse the adjacency list of v carrying on the dfs. *)
      p := G.AdjLists[v];
      while p <> nil do
        begin
          w := p^.id;
          if dfsnumber[w] = 0
            then begin
                   (* (v,w) is a tree edge; stack it and continue the dfs. *)
                   Push(v,w);
                   BiconnectedComponent(w,v);
                   (* On return, update highwater of the parent as needed. *)
                   if highwater[w] < highwater[v]
                     then highwater[v] := highwater[w];
                   if dfsnumber[v] <= highwater[w]
                     then (* Either v is the root of the tree or it is an
                             articulation point; all of the edges currently
                             on the stack above (v,w), together with (v,w),
                             form a biconnected component. *)
                          repeat
                            Pop(a,b);
                            ... (* handling of edge (a,b) *)
                          until (a = v) and (b = w)
                 end
            else if (dfsnumber[w] < dfsnumber[v]) and (w <> u)
                   then begin
                          (* (v,w) is a new back edge. *)
                          Push(v,w);
                          if dfsnumber[w] < highwater[v]
                            then highwater[v] := dfsnumber[w]
                        end;
          p := p^.next
        end
    end; (* BiconnectedComponent *)

  begin
    CreateStack;
    lastdfsnumber := 0;
    for i := 1 to G.size do dfsnumber[i] := 0;
    (* The graph is connected: one call, with any vertex, is sufficient. *)
    BiconnectedComponent(1,0 (* parent is nonexistent *))
  end; (* Biconnected *)
