(* The polygon is represented by a circular linked list. *)
type PtrToElement = ^ListElement;
     ListElement = record
                     vertex: point;
                     next: PtrToElement
                   end;

function Convex(polygon: PtrToElement): boolean;
  (* Assumptions: -- no consecutive collinear points or degenerate segments
                  -- polygon contains at least three vertices.
     NOT assumed: -- orientation, simplicity. *)
  label 99;
  var P1, previous, current, future: PtrToElement;
      (* previous, current, and future stand for P(k-1), P(k), and P(k+1). *)
      s, P1P2: segment;
      a, b, c: real;
  begin
    (* Initialize: set up line from P1 to P2 and the 3 traversal points. *)
    P1 := polygon; P1P2.e1 := P1^.vertex; P1P2.e2 := P1^.next^.vertex;
    previous := P1^.next; current := previous^.next; future := current^.next;

    (* Loop around the perimeter. *)
    while future <> P1 do
      (* Depending on orientation of polygon, future vertex must be either
           to the left of the oriented lines or
           to the right of the oriented lines
         from previous to current, from P1 to current, and from P1 to P2. *)
      begin
        (* Set up oriented line from previous to current and test. *)
        s.e1 := previous^.vertex; s.e2 := current^.vertex;
          a := WhichSide(future^.vertex,s);
        (* Set up oriented line from P1 to current and test. *)
        s.e1 := P1^.vertex;  (* s.e2 := current^.vertex; *)
          b := WhichSide(future^.vertex,s);
        (* Test against oriented line from P1 to P2. *)
          c := WhichSide(future^.vertex,P1P2);
        (* Absence of collinearity implies that a, b, and c are nonzero.
           The requirement on future is that a, b, and c have the same sign. *)
        if abs(a + b + c) <> abs(a) + abs(b) + abs(c)
          then begin
                 Convex := false;
                 goto 99
               end;
        previous := current; current := future; future:= future^.next
      end;
    Convex := true;
99: (* return *)
  end; (* Convex *)
