procedure DFS(var (* for speed *) G: graph);
  var i: vertex;
      visited: array [vertex] of boolean;
  
  (* Auxiliary recursive procedure RecursiveDFS does the work;
     a nonrecursive shell is needed because DFS must initialize visited[ ]
     and scan all the vertices to ensure traversing all the components
     and because visited[ ] must be global to retain values. *)
  procedure RecursiveDFS(i: vertex);
    var p: PtrToNode;
    begin
      (* Visit and mark vertex i... *)
      visit(i);
      visited[i] := true;

      (* ...and proceed recursively on to all unvisited neighbors of i. *)
      p := G.AdjLists[i];
      while p <> nil do
        begin
          if not visited[p^.id]
            then RecursiveDFS(p^.id);
          p := p^.next
        end
    end; (* RecursiveDFS *)

  begin
    (* Initially, no vertex has been visited. *)
    for i := 1 to G.size do visited[i] := false;

    (* Try each vertex in turn as the starting point;
       thus all components will be traversed. *)
    for i := 1 to G.size do
      if not visited[i] then RecursiveDFS(i)
  end; (* DFS *)
