(* Conventions:
     For ordering points--primary key is x, secondary key is y.
       less:     < 0
       equal:    = 0
       greater:  > 0

     For determining on which side of an oriented line a point lies
     or, phrased algebraically, whether a point falls in the positive
     or negative half plane determined by an oriented line.
       left or negative:   < 0
       on line:            = 0
       right or positive:  > 0

     For determining whether two points lie on the same or opposite sides
     of a line.
       same side:                           > 0
       one or both points lie on the line:  = 0
       opposite sides:                      < 0
*)

type point = record
               x, y: real
             end;
     segment = record
                 e1, e2: point (* endpoints of segment--no order implied *)
               end;

function Compare(p1, p2: point): real;
  (* Sort on x-coordinate, or in the event of a tie, y-coordinate. *)
  var comp: real;
  begin
    comp := p1.x - p2.x;
    if comp = 0
      then (* tie on x-coordinate, so use y-coordinate *)
           comp := p1.y - p2.y;
    Compare := comp
  end; (* Compare *)

function LesserOf(p1, p2: point): point;
  begin
    if Compare(p1,p2) <= 0
      then LesserOf := p1
      else LesserOf := p2
  end; (* LesserOf *)

function GreaterOf(p1, p2: point): point;
  begin
    if Compare(p1,p2) > 0
      then GreaterOf := p1
      else GreaterOf := p2
  end; (* GreaterOf *)
function WhichSide(p: point; s: segment): real;
  (* Given an oriented line from s.e1 passing through s.e2, determine on
     which side of the line the point p lies.  Alternatively, determine
     in which half plane, positive or negative, the point lies. *)
  (* If the segment degenerates to a point, then the point is always on it. *)
  var a, b: real;
  begin
    a := (p.x - s.e1.x)*(s.e2.y - s.e1.y);
    b := (p.y - s.e1.y)*(s.e2.x - s.e1.x);
    WhichSide := a - b
  end; (* WhichSide *)

function Sides(p1, p2: point; s: segment): real;
  (* Determine whether two points lie on the same side of a line. *)
  begin
    Sides := WhichSide(p1,s) * WhichSide(p2,s)
  end; (* Sides *)
