function Interior(p: point; polygon: PtrToElement): integer;
  (* Determines whether p lies inside (+1), on (0), or outside (-1)
     a simple polygon. *)
  (* Assumptions: -- polygon is simple and has at least 3 vertices.
     NOT assumed: -- orientation, absence of consecutive collinear vertices,
                     absence of degenerate segments. *)
  label 1, 99;
  const infinity = ...;  (* beyond any point's abscissa *)
  var previous, current, future: PtrToElement;
      parity: integer;
      edge, halfline: segment;
  begin
    (* Set up horizontal half line with endpoints p and infinity. *)
    halfline.e1 := p; halfline.e2.x := infinity; halfline.e2.y := p.y;
    (* Invariant: At most one of previous and current can lie on the half
       line.  Establish this condition initially by rotating through the
       polygon until the starting point lies on one or the other side of the
       half line.  Preconditions guarantee that this is always possible. *)
    while WhichSide(polygon^.vertex,halfline) = 0 do polygon := polygon^.next;
    parity := -1;  (* outside by default *)

    (* Prepare to walk around the perimeter--start with the second edge. *)
    previous := polygon; current := previous^.next; future := current^.next;

    (* The main loop walks the perimeter of polygon. *)
    repeat
   1: (* Set up current edge and test. *)
      edge.e1 := current^.vertex; edge.e2 := future^.vertex;
      if Intersect(edge,halfline)
        then if OnSegment(p,edge)
               then begin
                      Interior := 0;
                      goto 99 (* return *)
                    end
        else if OnSegment(edge.e1,halfline) and OnSegment(edge.e2,halfline)
               then (* The half line passes through both endpoints,
                       so shrink the edge to a single vertex. *)
                    begin
                      future := future^.next;
                      goto 1
                    end
        else if OnSegment(edge.e1,halfline)
               then (* The edge intersects the half line at the middle vertex
                       (current) only; determine whether we have a crossing. *)
                    begin
                      if Sides(previous^.vertex,future^.vertex,halfline) < 0
                         then parity := -parity
                    end
        else if not OnSegment(edge.e2,halfline)
               then parity := -parity;  (* clean crossing *)
     (* else the first endpoint is off and the second on the half line,
             so delay the decision until we see further vertices. *)

      (* Move around the perimeter. *)
      previous := current; current := future; future := future^.next
    until previous = polygon;
    Interior := parity;
99: (* return *)
  end; (* Interior *)
