function Intersect(s1, s2: segment): boolean;
  (* Determine whether two line segments, either or both of which can be
     degenerate, intersect. *)
  label 99;
  var sides1, sides2: real;
      max1, min1, max2, min2: point;
  begin
    sides1 := Sides(s2.e1,s2.e2,s1);
    sides2 := Sides(s1.e1,s1.e2,s2);
    (* If both endpoints of segment 2 lie on the same side of line 1
       or vice versa, there is no intersection--even if one segment
       degenerates to a point that does not lie on the line determined
       by the other segment. *)
    if (sides1 > 0) or (sides2 > 0)
      then begin
             Intersect := false;
             goto 99 (* return *)
           end;
    (* If the endpoints of one segment lie on opposite sides of the
       other, there is an intersection: segments form an `X' or a `T'. *)
    if sides1 + sides2 < 0
      then begin
             Intersect := true;
             goto 99 (* return *)
           end;
    (* Either the four points are collinear or the two segments form a `V'.
       Range comparisons handle even the degenerate cases of a point lying
       on a line or of two points. *)
    max1 := GreaterOf(s1.e1,s1.e2); min1 := LesserOf(s1.e1,s1.e2);
    max2 := GreaterOf(s2.e1,s2.e2); min2 := LesserOf(s2.e1,s2.e2);
    Intersect := (Compare(max2,min1) >= 0) and (Compare(max1,min2) >= 0);
99: (* return *)
  end; (* Intersect *)
