const TotalStorage = ...; (* MaxVertex * MaxVertex *)

var A: array [vertex, vertex] of ...;         (* adjacency matrix *)
    T: array [vertex, vertex] of integer;     (* time stamps *)
    InvT: array [1..TotalStorage] of integer; (* inverted list of time stamps *)
    n: integer;                               (* last time stamp assigned *)

procedure CreateLA;
  (* Initialize lazy array package--user visible. *)
  begin
    n := 0
  end; (* CreateLA *)

function Touched(i, j: vertex): boolean;
  (* Determine if A[i,j] has been initialized--not user visible. *)
  begin
    if (T[i,j] < 1) or (T[i,j] > n)
      then Touched := false (* random value out of range *)
    else if InvT[T[i,j]] <> MaxVertex*i + j
      then Touched := false (* random value within range *)
    else   Touched := true  (* cell initialized in the past *)
  end; (* Touched *)

procedure Touch(i, j: vertex);
  (* Mark A[i,j] initialized, if not already so marked--not user visible. *)
  begin
    if not Touched(i,j) (* If not already initialized... *)
      then begin
             (* ...it is now. *)
             n := n + 1; (* the clock ticks *)
             T[i,j] := n;
             InvT[n] := MaxVertex*i + j
           end
  end; (* Touch *)

function Fetch(i, j: vertex; var defined: boolean): ...;
  (* Return A[i,j] or indication A[i,j] is uninitialized--user visible. *)
  begin
    defined := Touched(i,j);
    Fetch := A[i,j]
  end; (* Fetch *)

procedure Store(x: ...; i, j: vertex);
  (* A[i,j] := x; user visible *)
  begin
    Touch(i,j); (* Harmless if A[i,j] already initialized. *)
    A[i,j] := x
  end; (* Store *)
