type Tsort = array [1..MaxVertex] of vertex;

procedure TopologicalSort(var (* for speed *) G: graph; var T: Tsort);
  (* Assumes that G is a directed acyclic graph. *)
  var i: vertex;
      n: integer; (* number of vertices currently output into T *)
      p: PtrToNode;
      incident: array [vertex] of integer; (* number of incident arcs *)

  (* Program assumes a stack package with operations
       procedure CreateStack, function IsEmptyStack: boolean,
       procedure Push(v: vertex), and procedure Pop(var v: vertex). *)

  begin
    (* Initialize count of number of arcs incident upon each vertex. *)
    for i := 1 to G.size do incident[i] := 0;
    for i := 1 to G.size do
      begin
        p := G.AdjLists[i];
        while p <> nil do
          begin
            incident[p^.id] := incident[p^.id] + 1;
            p := p^.next
          end
      end;

    (* Initialize "sources in reduced graph" to true sources. *)
    CreateStack;
    for i := 1 to G.size do
      if incident[i] = 0 then Push(i);

    n := 0;
    while not IsEmptyStack do
      begin
        (* Retrieve the next "source in reduced graph" and output it. *)
        Pop(i); n := n + 1; T[n] := i;

        (* Effectively remove vertex from graph. *)
        p := G.AdjLists[i];
        while p <> nil do
          begin
            incident[p^.id] := incident[p^.id] - 1;
            (* Is vertex a source in the reduced graph? *)
            if incident[p^.id] = 0 then Push(p^.id);
            p := p^.next
          end
      end
    (* if n <> G.size then graph contains a cycle. *)
  end; (* TopologicalSort *)
